@@ -1,20 +1,14 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-
 from collections import OrderedDict
+import types
 
 import numpy as np
 
+from matplotlib import cbook, rcParams
 from matplotlib.axes import Axes
 import matplotlib.axis as maxis
-from matplotlib import cbook
-from matplotlib import docstring
 import matplotlib.markers as mmarkers
 import matplotlib.patches as mpatches
 import matplotlib.path as mpath
-from matplotlib import rcParams
 import matplotlib.ticker as mticker
 import matplotlib.transforms as mtransforms
 import matplotlib.spines as mspines
@@ -49,42 +43,30 @@ class PolarTransform(mtransforms.Transform):
                         self._apply_theta_transforms))
 
     def transform_non_affine(self, tr):
-        xy = np.empty(tr.shape, float)
-
-        t = tr[:, 0:1]
-        r = tr[:, 1:2]
-        x = xy[:, 0:1]
-        y = xy[:, 1:2]
-
+        # docstring inherited
+        t, r = np.transpose(tr)
         # PolarAxes does not use the theta transforms here, but apply them for
         # backwards-compatibility if not being used by it.
         if self._apply_theta_transforms and self._axis is not None:
             t *= self._axis.get_theta_direction()
             t += self._axis.get_theta_offset()
-
         if self._use_rmin and self._axis is not None:
-            r = r - self._axis.get_rorigin()
-        mask = r < 0
-        x[:] = np.where(mask, np.nan, r * np.cos(t))
-        y[:] = np.where(mask, np.nan, r * np.sin(t))
-
-        return xy
-    transform_non_affine.__doc__ = \
-        mtransforms.Transform.transform_non_affine.__doc__
+            r = (r - self._axis.get_rorigin()) * self._axis.get_rsign()
+        r = np.where(r >= 0, r, np.nan)
+        return np.column_stack([r * np.cos(t), r * np.sin(t)])
 
     def transform_path_non_affine(self, path):
+        # docstring inherited
         vertices = path.vertices
         if len(vertices) == 2 and vertices[0, 0] == vertices[1, 0]:
             return mpath.Path(self.transform(vertices), path.codes)
         ipath = path.interpolated(path._interpolation_steps)
         return mpath.Path(self.transform(ipath.vertices), ipath.codes)
-    transform_path_non_affine.__doc__ = \
-        mtransforms.Transform.transform_path_non_affine.__doc__
 
     def inverted(self):
+        # docstring inherited
         return PolarAxes.InvertedPolarTransform(self._axis, self._use_rmin,
                                                 self._apply_theta_transforms)
-    inverted.__doc__ = mtransforms.Transform.inverted.__doc__
 
 
 class PolarAffine(mtransforms.Affine2DBase):
@@ -113,6 +95,7 @@ class PolarAffine(mtransforms.Affine2DBase):
                         mtransforms._indent_str(self._limits)))
 
     def get_matrix(self):
+        # docstring inherited
         if self._invalid:
             limits_scaled = self._limits.transformed(self._scale_transform)
             yscale = limits_scaled.ymax - limits_scaled.ymin
@@ -123,7 +106,6 @@ class PolarAffine(mtransforms.Affine2DBase):
             self._inverted = None
             self._invalid = 0
         return self._mtx
-    get_matrix.__doc__ = mtransforms.Affine2DBase.get_matrix.__doc__
 
 
 class InvertedPolarTransform(mtransforms.Transform):
@@ -153,17 +135,11 @@ class InvertedPolarTransform(mtransforms.Transform):
                         self._apply_theta_transforms))
 
     def transform_non_affine(self, xy):
+        # docstring inherited
         x = xy[:, 0:1]
         y = xy[:, 1:]
-        r = np.sqrt(x*x + y*y)
-        with np.errstate(invalid='ignore'):
-            # At x=y=r=0 this will raise an
-            # invalid value warning when doing 0/0
-            # Divide by zero warnings are only raised when
-            # the numerator is different from 0. That
-            # should not happen here.
-            theta = np.arccos(x / r)
-        theta = np.where(y < 0, 2 * np.pi - theta, theta)
+        r = np.hypot(x, y)
+        theta = (np.arctan2(y, x) + 2 * np.pi) % (2 * np.pi)
 
         # PolarAxes does not use the theta transforms here, but apply them for
         # backwards-compatibility if not being used by it.
@@ -174,15 +150,14 @@ class InvertedPolarTransform(mtransforms.Transform):
 
         if self._use_rmin and self._axis is not None:
             r += self._axis.get_rorigin()
+            r *= self._axis.get_rsign()
 
         return np.concatenate((theta, r), 1)
-    transform_non_affine.__doc__ = \
-        mtransforms.Transform.transform_non_affine.__doc__
 
     def inverted(self):
+        # docstring inherited
         return PolarAxes.PolarTransform(self._axis, self._use_rmin,
                                         self._apply_theta_transforms)
-    inverted.__doc__ = mtransforms.Transform.inverted.__doc__
 
 
 class ThetaFormatter(mticker.Formatter):
@@ -291,22 +266,22 @@ class ThetaTick(maxis.XTick):
         self._text2_translate = mtransforms.ScaledTranslation(
             0, 0,
             axes.figure.dpi_scale_trans)
-        super(ThetaTick, self).__init__(axes, *args, **kwargs)
+        super().__init__(axes, *args, **kwargs)
 
     def _get_text1(self):
-        t = super(ThetaTick, self)._get_text1()
+        t = super()._get_text1()
         t.set_rotation_mode('anchor')
         t.set_transform(t.get_transform() + self._text1_translate)
         return t
 
     def _get_text2(self):
-        t = super(ThetaTick, self)._get_text2()
+        t = super()._get_text2()
         t.set_rotation_mode('anchor')
         t.set_transform(t.get_transform() + self._text2_translate)
         return t
 
     def _apply_params(self, **kw):
-        super(ThetaTick, self)._apply_params(**kw)
+        super()._apply_params(**kw)
 
         # Ensure transform is correct; sometimes this gets reset.
         trans = self.label1.get_transform()
@@ -325,32 +300,31 @@ class ThetaTick(maxis.XTick):
         self._text2_translate.invalidate()
 
     def update_position(self, loc):
-        super(ThetaTick, self).update_position(loc)
+        super().update_position(loc)
         axes = self.axes
         angle = loc * axes.get_theta_direction() + axes.get_theta_offset()
         text_angle = np.rad2deg(angle) % 360 - 90
         angle -= np.pi / 2
 
-        if self.tick1On:
-            marker = self.tick1line.get_marker()
-            if marker in (mmarkers.TICKUP, '|'):
-                trans = mtransforms.Affine2D().scale(1.0, 1.0).rotate(angle)
-            elif marker == mmarkers.TICKDOWN:
-                trans = mtransforms.Affine2D().scale(1.0, -1.0).rotate(angle)
-            else:
-                # Don't modify custom tick line markers.
-                trans = self.tick1line._marker._transform
-            self.tick1line._marker._transform = trans
-        if self.tick2On:
-            marker = self.tick2line.get_marker()
-            if marker in (mmarkers.TICKUP, '|'):
-                trans = mtransforms.Affine2D().scale(1.0, 1.0).rotate(angle)
-            elif marker == mmarkers.TICKDOWN:
-                trans = mtransforms.Affine2D().scale(1.0, -1.0).rotate(angle)
-            else:
-                # Don't modify custom tick line markers.
-                trans = self.tick2line._marker._transform
-            self.tick2line._marker._transform = trans
+        marker = self.tick1line.get_marker()
+        if marker in (mmarkers.TICKUP, '|'):
+            trans = mtransforms.Affine2D().scale(1, 1).rotate(angle)
+        elif marker == mmarkers.TICKDOWN:
+            trans = mtransforms.Affine2D().scale(1, -1).rotate(angle)
+        else:
+            # Don't modify custom tick line markers.
+            trans = self.tick1line._marker._transform
+        self.tick1line._marker._transform = trans
+
+        marker = self.tick2line.get_marker()
+        if marker in (mmarkers.TICKUP, '|'):
+            trans = mtransforms.Affine2D().scale(1, 1).rotate(angle)
+        elif marker == mmarkers.TICKDOWN:
+            trans = mtransforms.Affine2D().scale(1, -1).rotate(angle)
+        else:
+            # Don't modify custom tick line markers.
+            trans = self.tick2line._marker._transform
+        self.tick2line._marker._transform = trans
 
         mode, user_angle = self._labelrotation
         if mode == 'default':
@@ -361,10 +335,8 @@ class ThetaTick(maxis.XTick):
             elif text_angle < -90:
                 text_angle += 180
             text_angle += user_angle
-        if self.label1On:
-            self.label1.set_rotation(text_angle)
-        if self.label2On:
-            self.label2.set_rotation(text_angle)
+        self.label1.set_rotation(text_angle)
+        self.label2.set_rotation(text_angle)
 
         # This extra padding helps preserve the look from previous releases but
         # is also needed because labels are anchored to their center.
@@ -398,19 +370,19 @@ class ThetaAxis(maxis.XAxis):
         self.isDefault_majfmt = True
 
     def cla(self):
-        super(ThetaAxis, self).cla()
+        super().cla()
         self.set_ticks_position('none')
         self._wrap_locator_formatter()
 
     def _set_scale(self, value, **kwargs):
-        super(ThetaAxis, self)._set_scale(value, **kwargs)
+        super()._set_scale(value, **kwargs)
         self._wrap_locator_formatter()
 
     def _copy_tick_props(self, src, dest):
         'Copy the props from src tick to dest tick'
         if src is None or dest is None:
             return
-        super(ThetaAxis, self)._copy_tick_props(src, dest)
+        super()._copy_tick_props(src, dest)
 
         # Ensure that tick transforms are independent so that padding works.
         trans = dest._get_text1_transform()[0]
@@ -437,10 +409,9 @@ class RadialLocator(mticker.Locator):
         # Ensure previous behaviour with full circle non-annular views.
         if self._axes:
             if _is_full_circle_rad(*self._axes.viewLim.intervalx):
-                rorigin = self._axes.get_rorigin()
+                rorigin = self._axes.get_rorigin() * self._axes.get_rsign()
                 if self._axes.get_rmin() <= rorigin:
                     show_all = False
-
         if show_all:
             return self.base()
         else:
@@ -460,7 +431,10 @@ class RadialLocator(mticker.Locator):
 
     def view_limits(self, vmin, vmax):
         vmin, vmax = self.base.view_limits(vmin, vmax)
-        return mtransforms.nonsingular(min(0, vmin), vmax)
+        if vmax > vmin:
+            # this allows inverted r/y-lims
+            vmin = min(0, vmin)
+        return mtransforms.nonsingular(vmin, vmax)
 
 
 class _ThetaShift(mtransforms.ScaledTranslation):
@@ -533,12 +507,12 @@ class RadialTick(maxis.YTick):
     enabled.
     """
     def _get_text1(self):
-        t = super(RadialTick, self)._get_text1()
+        t = super()._get_text1()
         t.set_rotation_mode('anchor')
         return t
 
     def _get_text2(self):
-        t = super(RadialTick, self)._get_text2()
+        t = super()._get_text2()
         t.set_rotation_mode('anchor')
         return t
 
@@ -597,7 +571,7 @@ class RadialTick(maxis.YTick):
                     return 'center', 'bottom'
 
     def update_position(self, loc):
-        super(RadialTick, self).update_position(loc)
+        super().update_position(loc)
         axes = self.axes
         thetamin = axes.get_thetamin()
         thetamax = axes.get_thetamax()
@@ -610,100 +584,71 @@ class RadialTick(maxis.YTick):
             angle = (axes.get_rlabel_position() * direction +
                      offset) % 360 - 90
             tick_angle = 0
-            if angle > 90:
-                text_angle = angle - 180
-            elif angle < -90:
-                text_angle = angle + 180
-            else:
-                text_angle = angle
         else:
             angle = (thetamin * direction + offset) % 360 - 90
             if direction > 0:
                 tick_angle = np.deg2rad(angle)
             else:
                 tick_angle = np.deg2rad(angle + 180)
-            if angle > 90:
-                text_angle = angle - 180
-            elif angle < -90:
-                text_angle = angle + 180
-            else:
-                text_angle = angle
+        text_angle = (angle + 90) % 180 - 90  # between -90 and +90.
         mode, user_angle = self._labelrotation
         if mode == 'auto':
             text_angle += user_angle
         else:
             text_angle = user_angle
-        if self.label1On:
-            if full:
-                ha = 'left'
-                va = 'bottom'
-            else:
-                ha, va = self._determine_anchor(mode, angle, direction > 0)
-            self.label1.set_ha(ha)
-            self.label1.set_va(va)
-            self.label1.set_rotation(text_angle)
-        if self.tick1On:
-            marker = self.tick1line.get_marker()
-            if marker == mmarkers.TICKLEFT:
-                trans = (mtransforms.Affine2D()
-                         .scale(1.0, 1.0)
-                         .rotate(tick_angle))
-            elif marker == '_':
-                trans = (mtransforms.Affine2D()
-                         .scale(1.0, 1.0)
-                         .rotate(tick_angle + np.pi / 2))
-            elif marker == mmarkers.TICKRIGHT:
-                trans = (mtransforms.Affine2D()
-                         .scale(-1.0, 1.0)
-                         .rotate(tick_angle))
-            else:
-                # Don't modify custom tick line markers.
-                trans = self.tick1line._marker._transform
-            self.tick1line._marker._transform = trans
 
         if full:
-            self.label2On = False
-            self.tick2On = False
+            ha = self.label1.get_horizontalalignment()
+            va = self.label1.get_verticalalignment()
         else:
-            angle = (thetamax * direction + offset) % 360 - 90
-            if direction > 0:
-                tick_angle = np.deg2rad(angle)
-            else:
-                tick_angle = np.deg2rad(angle + 180)
-            if angle > 90:
-                text_angle = angle - 180
-            elif angle < -90:
-                text_angle = angle + 180
-            else:
-                text_angle = angle
+            ha, va = self._determine_anchor(mode, angle, direction > 0)
+        self.label1.set_horizontalalignment(ha)
+        self.label1.set_verticalalignment(va)
+        self.label1.set_rotation(text_angle)
+
+        marker = self.tick1line.get_marker()
+        if marker == mmarkers.TICKLEFT:
+            trans = mtransforms.Affine2D().rotate(tick_angle)
+        elif marker == '_':
+            trans = mtransforms.Affine2D().rotate(tick_angle + np.pi / 2)
+        elif marker == mmarkers.TICKRIGHT:
+            trans = mtransforms.Affine2D().scale(-1, 1).rotate(tick_angle)
+        else:
+            # Don't modify custom tick line markers.
+            trans = self.tick1line._marker._transform
+        self.tick1line._marker._transform = trans
+
+        if full:
+            self.label2.set_visible(False)
+            self.tick2line.set_visible(False)
+        angle = (thetamax * direction + offset) % 360 - 90
+        if direction > 0:
+            tick_angle = np.deg2rad(angle)
+        else:
+            tick_angle = np.deg2rad(angle + 180)
+        text_angle = (angle + 90) % 180 - 90  # between -90 and +90.
         mode, user_angle = self._labelrotation
         if mode == 'auto':
             text_angle += user_angle
         else:
             text_angle = user_angle
-        if self.label2On:
-            ha, va = self._determine_anchor(mode, angle, direction < 0)
-            self.label2.set_ha(ha)
-            self.label2.set_va(va)
-            self.label2.set_rotation(text_angle)
-        if self.tick2On:
-            marker = self.tick2line.get_marker()
-            if marker == mmarkers.TICKLEFT:
-                trans = (mtransforms.Affine2D()
-                         .scale(1.0, 1.0)
-                         .rotate(tick_angle))
-            elif marker == '_':
-                trans = (mtransforms.Affine2D()
-                         .scale(1.0, 1.0)
-                         .rotate(tick_angle + np.pi / 2))
-            elif marker == mmarkers.TICKRIGHT:
-                trans = (mtransforms.Affine2D()
-                         .scale(-1.0, 1.0)
-                         .rotate(tick_angle))
-            else:
-                # Don't modify custom tick line markers.
-                trans = self.tick2line._marker._transform
-            self.tick2line._marker._transform = trans
+
+        ha, va = self._determine_anchor(mode, angle, direction < 0)
+        self.label2.set_ha(ha)
+        self.label2.set_va(va)
+        self.label2.set_rotation(text_angle)
+
+        marker = self.tick2line.get_marker()
+        if marker == mmarkers.TICKLEFT:
+            trans = mtransforms.Affine2D().rotate(tick_angle)
+        elif marker == '_':
+            trans = mtransforms.Affine2D().rotate(tick_angle + np.pi / 2)
+        elif marker == mmarkers.TICKRIGHT:
+            trans = mtransforms.Affine2D().scale(-1, 1).rotate(tick_angle)
+        else:
+            # Don't modify custom tick line markers.
+            trans = self.tick2line._marker._transform
+        self.tick2line._marker._transform = trans
 
 
 class RadialAxis(maxis.YAxis):
@@ -717,7 +662,7 @@ class RadialAxis(maxis.YAxis):
     axis_name = 'radius'
 
     def __init__(self, *args, **kwargs):
-        super(RadialAxis, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.sticky_edges.y.append(0)
 
     def _get_tick(self, major):
@@ -733,12 +678,12 @@ class RadialAxis(maxis.YAxis):
         self.isDefault_majloc = True
 
     def cla(self):
-        super(RadialAxis, self).cla()
+        super().cla()
         self.set_ticks_position('none')
         self._wrap_locator_formatter()
 
     def _set_scale(self, value, **kwargs):
-        super(RadialAxis, self)._set_scale(value, **kwargs)
+        super()._set_scale(value, **kwargs)
         self._wrap_locator_formatter()
 
 
@@ -766,7 +711,7 @@ class _WedgeBbox(mtransforms.Bbox):
 
     Parameters
     ----------
-    center : tuple of float
+    center : (float, float)
         Center of the wedge
     viewLim : `~matplotlib.transforms.Bbox`
         Bbox determining the boundaries of the wedge
@@ -793,9 +738,9 @@ class _WedgeBbox(mtransforms.Bbox):
                         mtransforms._indent_str(self._originLim)))
 
     def get_points(self):
+        # docstring inherited
         if self._invalid:
             points = self._viewLim.get_points().copy()
-
             # Scale angular limits to work with Wedge.
             points[:, 0] *= 180 / np.pi
             if points[0, 0] > points[1, 0]:
@@ -817,21 +762,13 @@ class _WedgeBbox(mtransforms.Bbox):
 
             # Ensure equal aspect ratio.
             w, h = self._points[1] - self._points[0]
-            if h < w:
-                deltah = (w - h) / 2.0
-                deltaw = 0.0
-            elif w < h:
-                deltah = 0.0
-                deltaw = (h - w) / 2.0
-            else:
-                deltah = 0.0
-                deltaw = 0.0
+            deltah = max(w - h, 0) / 2
+            deltaw = max(h - w, 0) / 2
             self._points += np.array([[-deltaw, -deltah], [deltaw, deltah]])
 
             self._invalid = 0
 
         return self._points
-    get_points.__doc__ = mtransforms.Bbox.get_points.__doc__
 
 
 class PolarAxes(Axes):
@@ -842,20 +779,17 @@ class PolarAxes(Axes):
     """
     name = 'polar'
 
-    def __init__(self, *args, **kwargs):
-        """
-        Create a new Polar Axes for a polar plot.
-        """
-        self._default_theta_offset = kwargs.pop('theta_offset', 0)
-        self._default_theta_direction = kwargs.pop('theta_direction', 1)
-        self._default_rlabel_position = np.deg2rad(
-            kwargs.pop('rlabel_position', 22.5))
-
-        Axes.__init__(self, *args, **kwargs)
+    def __init__(self, *args,
+                 theta_offset=0, theta_direction=1, rlabel_position=22.5,
+                 **kwargs):
+        # docstring inherited
+        self._default_theta_offset = theta_offset
+        self._default_theta_direction = theta_direction
+        self._default_rlabel_position = np.deg2rad(rlabel_position)
+        super().__init__(*args, **kwargs)
         self.use_sticky_edges = True
         self.set_aspect('equal', adjustable='box', anchor='C')
         self.cla()
-    __init__.__doc__ = Axes.__init__.__doc__
 
     def cla(self):
         Axes.cla(self)
@@ -974,9 +908,7 @@ class PolarAxes(Axes):
             self._r_label_position + self.transData)
 
     def get_xaxis_transform(self, which='grid'):
-        if which not in ['tick1', 'tick2', 'grid']:
-            raise ValueError(
-                "'which' must be one of 'tick1', 'tick2', or 'grid'")
+        cbook._check_in_list(['tick1', 'tick2', 'grid'], which=which)
         return self._xaxis_transform
 
     def get_xaxis_text1_transform(self, pad):
@@ -991,8 +923,7 @@ class PolarAxes(Axes):
         elif which == 'grid':
             return self._yaxis_transform
         else:
-            raise ValueError(
-                "'which' must be one of 'tick1', 'tick2', or 'grid'")
+            cbook._check_in_list(['tick1', 'tick2', 'grid'], which=which)
 
     def get_yaxis_text1_transform(self, pad):
         thetamin, thetamax = self._realViewLim.intervalx
@@ -1019,8 +950,8 @@ class PolarAxes(Axes):
         thetamin, thetamax = np.rad2deg(self._realViewLim.intervalx)
         if thetamin > thetamax:
             thetamin, thetamax = thetamax, thetamin
-        rmin, rmax = self._realViewLim.intervaly - self.get_rorigin()
-
+        rmin, rmax = ((self._realViewLim.intervaly - self.get_rorigin()) *
+                        self.get_rsign())
         if isinstance(self.patch, mpatches.Wedge):
             # Backwards-compatibility: Any subclassed Axes might override the
             # patch to not be the Wedge that PolarAxes uses.
@@ -1146,15 +1077,14 @@ class PolarAxes(Axes):
            Theta increases in the counterclockwise direction
         """
         mtx = self._direction.get_matrix()
-        if direction in ('clockwise',):
+        if direction in ('clockwise', -1):
             mtx[0, 0] = -1
-        elif direction in ('counterclockwise', 'anticlockwise'):
+        elif direction in ('counterclockwise', 'anticlockwise', 1):
             mtx[0, 0] = 1
-        elif direction in (1, -1):
-            mtx[0, 0] = direction
         else:
-            raise ValueError(
-                "direction must be 1, -1, clockwise or counterclockwise")
+            cbook._check_in_list(
+                [-1, 1, 'clockwise', 'counterclockwise', 'anticlockwise'],
+                direction=direction)
         self._direction.invalidate()
 
     def get_theta_direction(self):
@@ -1187,12 +1117,79 @@ class PolarAxes(Axes):
     def get_rorigin(self):
         return self._originViewLim.y0
 
-    def set_rlim(self, *args, **kwargs):
+    def get_rsign(self):
+        return np.sign(self._originViewLim.y1 - self._originViewLim.y0)
+
+    def set_rlim(self, bottom=None, top=None, emit=True, auto=False, **kwargs):
+        """
+        See `~.polar.PolarAxes.set_ylim`.
+        """
         if 'rmin' in kwargs:
-            kwargs['ymin'] = kwargs.pop('rmin')
+            if bottom is None:
+                bottom = kwargs.pop('rmin')
+            else:
+                raise ValueError('Cannot supply both positional "bottom"'
+                                 'argument and kwarg "rmin"')
         if 'rmax' in kwargs:
-            kwargs['ymax'] = kwargs.pop('rmax')
-        return self.set_ylim(*args, **kwargs)
+            if top is None:
+                top = kwargs.pop('rmax')
+            else:
+                raise ValueError('Cannot supply both positional "top"'
+                                 'argument and kwarg "rmax"')
+        return self.set_ylim(bottom=bottom, top=top, emit=emit, auto=auto,
+                             **kwargs)
+
+    def set_ylim(self, bottom=None, top=None, emit=True, auto=False,
+                 *, ymin=None, ymax=None):
+        """
+        Set the data limits for the radial axis.
+
+        Parameters
+        ----------
+        bottom : scalar, optional
+            The bottom limit (default: None, which leaves the bottom
+            limit unchanged).
+            The bottom and top ylims may be passed as the tuple
+            (*bottom*, *top*) as the first positional argument (or as
+            the *bottom* keyword argument).
+
+        top : scalar, optional
+            The top limit (default: None, which leaves the top limit
+            unchanged).
+
+        emit : bool, optional
+            Whether to notify observers of limit change (default: True).
+
+        auto : bool or None, optional
+            Whether to turn on autoscaling of the y-axis. True turns on,
+            False turns off (default action), None leaves unchanged.
+
+        ymin, ymax : scalar, optional
+            These arguments are deprecated and will be removed in a future
+            version.  They are equivalent to *bottom* and *top* respectively,
+            and it is an error to pass both *ymin* and *bottom* or
+            *ymax* and *top*.
+
+        Returns
+        -------
+        bottom, top : (float, float)
+            The new y-axis limits in data coordinates.
+        """
+        if ymin is not None:
+            if bottom is not None:
+                raise ValueError('Cannot supply both positional "bottom" '
+                                 'argument and kwarg "ymin"')
+            else:
+                bottom = ymin
+        if ymax is not None:
+            if top is not None:
+                raise ValueError('Cannot supply both positional "top" '
+                                 'argument and kwarg "ymax"')
+            else:
+                top = ymax
+        if top is None and np.iterable(bottom):
+            bottom, top = bottom[0], bottom[1]
+        return super().set_ylim(bottom=bottom, top=top, emit=emit, auto=auto)
 
     def get_rlabel_position(self):
         """
@@ -1224,37 +1221,40 @@ class PolarAxes(Axes):
     def set_rticks(self, *args, **kwargs):
         return Axes.set_yticks(self, *args, **kwargs)
 
-    @docstring.dedent_interpd
-    def set_thetagrids(self, angles, labels=None, frac=None, fmt=None,
-                       **kwargs):
+    def set_thetagrids(self, angles, labels=None, fmt=None, **kwargs):
         """
-        Set the angles at which to place the theta grids (these
-        gridlines are equal along the theta dimension).  *angles* is in
-        degrees.
+        Set the theta gridlines in a polar plot.
 
-        *labels*, if not None, is a ``len(angles)`` list of strings of
-        the labels to use at each angle.
+        Parameters
+        ----------
+        angles : tuple with floats, degrees
+            The angles of the theta gridlines.
 
-        If *labels* is None, the labels will be ``fmt %% angle``
+        labels : tuple with strings or None
+            The labels to use at each theta gridline. The
+            `.projections.polar.ThetaFormatter` will be used if None.
 
-        *frac* is the fraction of the polar axes radius at which to
-        place the label (1 is the edge). e.g., 1.05 is outside the axes
-        and 0.95 is inside the axes.
+        fmt : str or None
+            Format string used in `matplotlib.ticker.FormatStrFormatter`.
+            For example '%f'. Note that the angle that is used is in
+            radians.
 
-        Return value is a list of tuples (*line*, *label*), where
-        *line* is :class:`~matplotlib.lines.Line2D` instances and the
-        *label* is :class:`~matplotlib.text.Text` instances.
+        Returns
+        -------
+        lines, labels : list of `.lines.Line2D`, list of `.text.Text`
+            *lines* are the theta gridlines and *labels* are the tick labels.
 
-        kwargs are optional text properties for the labels:
+        Other Parameters
+        ----------------
+        **kwargs
+            *kwargs* are optional `~.Text` properties for the labels.
 
-        %(Text)s
-
-        ACCEPTS: sequence of floats
+        See Also
+        --------
+        .PolarAxes.set_rgrids
+        .Axis.get_gridlines
+        .Axis.get_ticklabels
         """
-        if frac is not None:
-            cbook.warn_deprecated('2.1', name='frac', obj_type='parameter',
-                                  alternative='tick padding via '
-                                              'Axes.tick_params')
 
         # Make sure we take into account unitized data
         angles = self.convert_yunits(angles)
@@ -1268,29 +1268,42 @@ class PolarAxes(Axes):
             t.update(kwargs)
         return self.xaxis.get_ticklines(), self.xaxis.get_ticklabels()
 
-    @docstring.dedent_interpd
     def set_rgrids(self, radii, labels=None, angle=None, fmt=None,
                    **kwargs):
         """
-        Set the radial locations and labels of the *r* grids.
+        Set the radial gridlines on a polar plot.
 
-        The labels will appear at radial distances *radii* at the
-        given *angle* in degrees.
+        Parameters
+        ----------
+        radii : tuple with floats
+            The radii for the radial gridlines
 
-        *labels*, if not None, is a ``len(radii)`` list of strings of the
-        labels to use at each radius.
+        labels : tuple with strings or None
+            The labels to use at each radial gridline. The
+            `matplotlib.ticker.ScalarFormatter` will be used if None.
 
-        If *labels* is None, the built-in formatter will be used.
+        angle : float
+            The angular position of the radius labels in degrees.
 
-        Return value is a list of tuples (*line*, *label*), where
-        *line* is :class:`~matplotlib.lines.Line2D` instances and the
-        *label* is :class:`~matplotlib.text.Text` instances.
+        fmt : str or None
+            Format string used in `matplotlib.ticker.FormatStrFormatter`.
+            For example '%f'.
 
-        kwargs are optional text properties for the labels:
+        Returns
+        -------
+        lines, labels : list of `.lines.Line2D`, list of `.text.Text`
+            *lines* are the radial gridlines and *labels* are the tick labels.
 
-        %(Text)s
+        Other Parameters
+        ----------------
+        **kwargs
+            *kwargs* are optional `~.Text` properties for the labels.
 
-        ACCEPTS: sequence of floats
+        See Also
+        --------
+        .PolarAxes.set_thetagrids
+        .Axis.get_gridlines
+        .Axis.get_ticklabels
         """
         # Make sure we take into account unitized data
         radii = self.convert_xunits(radii)
@@ -1357,12 +1370,12 @@ class PolarAxes(Axes):
         if button == 1:
             epsilon = np.pi / 45.0
             t, r = self.transData.inverted().transform_point((x, y))
-            if t >= angle - epsilon and t <= angle + epsilon:
+            if angle - epsilon <= t <= angle + epsilon:
                 mode = 'drag_r_labels'
         elif button == 3:
             mode = 'zoom'
 
-        self._pan_start = cbook.Bunch(
+        self._pan_start = types.SimpleNamespace(
             rmax=self.get_rmax(),
             trans=self.transData.frozen(),
             trans_inverse=self.transData.inverted().frozen(),
@@ -1417,121 +1430,3 @@ PolarAxes.InvertedPolarTransform = InvertedPolarTransform
 PolarAxes.ThetaFormatter = ThetaFormatter
 PolarAxes.RadialLocator = RadialLocator
 PolarAxes.ThetaLocator = ThetaLocator
-
-
-# These are a couple of aborted attempts to project a polar plot using
-# cubic bezier curves.
-
-#         def transform_path(self, path):
-#             twopi = 2.0 * np.pi
-#             halfpi = 0.5 * np.pi
-
-#             vertices = path.vertices
-#             t0 = vertices[0:-1, 0]
-#             t1 = vertices[1:  , 0]
-#             td = np.where(t1 > t0, t1 - t0, twopi - (t0 - t1))
-#             maxtd = td.max()
-#             interpolate = np.ceil(maxtd / halfpi)
-#             if interpolate > 1.0:
-#                 vertices = self.interpolate(vertices, interpolate)
-
-#             vertices = self.transform(vertices)
-
-#             result = np.zeros((len(vertices) * 3 - 2, 2), float)
-#             codes = mpath.Path.CURVE4 * np.ones((len(vertices) * 3 - 2, ),
-#                                                 mpath.Path.code_type)
-#             result[0] = vertices[0]
-#             codes[0] = mpath.Path.MOVETO
-
-#             kappa = 4.0 * ((np.sqrt(2.0) - 1.0) / 3.0)
-#             kappa = 0.5
-
-#             p0   = vertices[0:-1]
-#             p1   = vertices[1:  ]
-
-#             x0   = p0[:, 0:1]
-#             y0   = p0[:, 1: ]
-#             b0   = ((y0 - x0) - y0) / ((x0 + y0) - x0)
-#             a0   = y0 - b0*x0
-
-#             x1   = p1[:, 0:1]
-#             y1   = p1[:, 1: ]
-#             b1   = ((y1 - x1) - y1) / ((x1 + y1) - x1)
-#             a1   = y1 - b1*x1
-
-#             x = -(a0-a1) / (b0-b1)
-#             y = a0 + b0*x
-
-#             xk = (x - x0) * kappa + x0
-#             yk = (y - y0) * kappa + y0
-
-#             result[1::3, 0:1] = xk
-#             result[1::3, 1: ] = yk
-
-#             xk = (x - x1) * kappa + x1
-#             yk = (y - y1) * kappa + y1
-
-#             result[2::3, 0:1] = xk
-#             result[2::3, 1: ] = yk
-
-#             result[3::3] = p1
-
-#             print(vertices[-2:])
-#             print(result[-2:])
-
-#             return mpath.Path(result, codes)
-
-#             twopi = 2.0 * np.pi
-#             halfpi = 0.5 * np.pi
-
-#             vertices = path.vertices
-#             t0 = vertices[0:-1, 0]
-#             t1 = vertices[1:  , 0]
-#             td = np.where(t1 > t0, t1 - t0, twopi - (t0 - t1))
-#             maxtd = td.max()
-#             interpolate = np.ceil(maxtd / halfpi)
-
-#             print("interpolate", interpolate)
-#             if interpolate > 1.0:
-#                 vertices = self.interpolate(vertices, interpolate)
-
-#             result = np.zeros((len(vertices) * 3 - 2, 2), float)
-#             codes = mpath.Path.CURVE4 * np.ones((len(vertices) * 3 - 2, ),
-#                                                 mpath.Path.code_type)
-#             result[0] = vertices[0]
-#             codes[0] = mpath.Path.MOVETO
-
-#             kappa = 4.0 * ((np.sqrt(2.0) - 1.0) / 3.0)
-#             tkappa = np.arctan(kappa)
-#             hyp_kappa = np.sqrt(kappa*kappa + 1.0)
-
-#             t0 = vertices[0:-1, 0]
-#             t1 = vertices[1:  , 0]
-#             r0 = vertices[0:-1, 1]
-#             r1 = vertices[1:  , 1]
-
-#             td = np.where(t1 > t0, t1 - t0, twopi - (t0 - t1))
-#             td_scaled = td / (np.pi * 0.5)
-#             rd = r1 - r0
-#             r0kappa = r0 * kappa * td_scaled
-#             r1kappa = r1 * kappa * td_scaled
-#             ravg_kappa = ((r1 + r0) / 2.0) * kappa * td_scaled
-
-#             result[1::3, 0] = t0 + (tkappa * td_scaled)
-#             result[1::3, 1] = r0*hyp_kappa
-#             # result[1::3, 1] = r0 / np.cos(tkappa * td_scaled)
-#             # np.sqrt(r0*r0 + ravg_kappa*ravg_kappa)
-
-#             result[2::3, 0] = t1 - (tkappa * td_scaled)
-#             result[2::3, 1] = r1*hyp_kappa
-#             # result[2::3, 1] = r1 / np.cos(tkappa * td_scaled)
-#             # np.sqrt(r1*r1 + ravg_kappa*ravg_kappa)
-
-#             result[3::3, 0] = t1
-#             result[3::3, 1] = r1
-
-#             print(vertices[:6], result[:6], t0[:6], t1[:6], td[:6],
-#                   td_scaled[:6], tkappa)
-#             result = self.transform(result)
-#             return mpath.Path(result, codes)
-#         transform_path_non_affine = transform_path
