@@ -2,45 +2,73 @@
 # Parts rewritten by Reinier Heeres <reinier@heeres.eu>
 # Minor additions by Ben Axelrod <baxelrod@coroware.com>
 
-'''
+"""
 Module containing 3D artist code and functions to convert 2D
 artists into 3D versions which can be added to an Axes3D.
-'''
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+"""
 
-import six
-from six.moves import zip
-
-from matplotlib import lines, text as mtext, path as mpath, colors as mcolors
-from matplotlib import artist
-from matplotlib.collections import Collection, LineCollection, \
-        PolyCollection, PatchCollection, PathCollection
-from matplotlib.cm import ScalarMappable
-from matplotlib.patches import Patch
-from matplotlib.colors import Normalize
-from matplotlib.cbook import iterable
-
-import warnings
-import numpy as np
 import math
+
+import numpy as np
+
+from matplotlib import (
+    artist, cbook, colors as mcolors, lines, text as mtext, path as mpath)
+from matplotlib.collections import (
+    LineCollection, PolyCollection, PatchCollection, PathCollection)
+from matplotlib.colors import Normalize
+from matplotlib.patches import Patch
 from . import proj3d
 
-def norm_angle(a):
-    """Return angle between -180 and +180"""
+
+def _norm_angle(a):
+    """Return the given angle normalized to -180 < *a* <= 180 degrees."""
     a = (a + 360) % 360
     if a > 180:
         a = a - 360
     return a
 
-def norm_text_angle(a):
-    """Return angle between -90 and +90"""
+
+@cbook.deprecated("3.1")
+def norm_angle(a):
+    """Return the given angle normalized to -180 < *a* <= 180 degrees."""
+    return _norm_angle(a)
+
+
+def _norm_text_angle(a):
+    """Return the given angle normalized to -90 < *a* <= 90 degrees."""
     a = (a + 180) % 180
     if a > 90:
         a = a - 180
     return a
 
+
+@cbook.deprecated("3.1")
+def norm_text_angle(a):
+    """Return the given angle normalized to -90 < *a* <= 90 degrees."""
+    return _norm_text_angle(a)
+
+
 def get_dir_vector(zdir):
+    """
+    Return a direction vector.
+
+    Parameters
+    ----------
+    zdir : {'x', 'y', 'z', None, 3-tuple}
+        The direction. Possible values are:
+        - 'x': equivalent to (1, 0, 0)
+        - 'y': equivalent to (0, 1, 0)
+        - 'z': equivalent to (0, 0, 1)
+        - *None*: equivalent to (0, 0, 0)
+        - an iterable (x, y, z) is returned unchanged.
+
+    Returns
+    -------
+    x, y, z : array-like
+        The direction vector. This is either a numpy.array or *zdir* itself if
+        *zdir* is already a length-3 iterable.
+
+    """
     if zdir == 'x':
         return np.array((1, 0, 0))
     elif zdir == 'y':
@@ -49,24 +77,33 @@ def get_dir_vector(zdir):
         return np.array((0, 0, 1))
     elif zdir is None:
         return np.array((0, 0, 0))
-    elif iterable(zdir) and len(zdir) == 3:
+    elif np.iterable(zdir) and len(zdir) == 3:
         return zdir
     else:
         raise ValueError("'x', 'y', 'z', None or vector of length 3 expected")
 
+
 class Text3D(mtext.Text):
-    '''
-    Text object with 3D position and (in the future) direction.
-    '''
+    """
+    Text object with 3D position and direction.
+
+    Parameters
+    ----------
+    x, y, z
+        The position of the text.
+    text : str
+        The text string to display.
+    zdir : {'x', 'y', 'z', None, 3-tuple}
+        The direction of the text. See `.get_dir_vector` for a description of
+        the values.
+
+    Other Parameters
+    ----------------
+    **kwargs
+         All other parameters are passed on to `~matplotlib.text.Text`.
+   """
 
     def __init__(self, x=0, y=0, z=0, text='', zdir='z', **kwargs):
-        '''
-        *x*, *y*, *z*  Position of text
-        *text*         Text string to display
-        *zdir*         Direction of text
-
-        Keyword arguments are passed onto :func:`~matplotlib.text.Text`.
-        '''
         mtext.Text.__init__(self, x, y, text, **kwargs)
         self.set_3d_properties(z, zdir)
 
@@ -76,21 +113,23 @@ class Text3D(mtext.Text):
         self._dir_vec = get_dir_vector(zdir)
         self.stale = True
 
+    @artist.allow_rasterization
     def draw(self, renderer):
-        proj = proj3d.proj_trans_points([self._position3d, \
-                self._position3d + self._dir_vec], renderer.M)
+        proj = proj3d.proj_trans_points(
+            [self._position3d, self._position3d + self._dir_vec], renderer.M)
         dx = proj[0][1] - proj[0][0]
         dy = proj[1][1] - proj[1][0]
-        if dx==0. and dy==0.:
-            # atan2 raises ValueError: math domain error on 0,0
-            angle = 0.
-        else:
-            angle = math.degrees(math.atan2(dy, dx))
+        angle = math.degrees(math.atan2(dy, dx))
         self.set_position((proj[0][0], proj[1][0]))
-        self.set_rotation(norm_text_angle(angle))
+        self.set_rotation(_norm_text_angle(angle))
         mtext.Text.draw(self, renderer)
         self.stale = False
 
+    def get_tightbbox(self, renderer):
+        # Overwriting the 2d Text behavior which is not valid for 3d.
+        # For now, just return None to exclude from layout calculation.
+        return None
+
 
 def text_2d_to_3d(obj, z=0, zdir='z'):
     """Convert a Text to a Text3D object."""
@@ -99,14 +138,14 @@ def text_2d_to_3d(obj, z=0, zdir='z'):
 
 
 class Line3D(lines.Line2D):
-    '''
+    """
     3D line object.
-    '''
+    """
 
     def __init__(self, xs, ys, zs, *args, **kwargs):
-        '''
+        """
         Keyword arguments are passed onto :func:`~matplotlib.lines.Line2D`.
-        '''
+        """
         lines.Line2D.__init__(self, [], [], *args, **kwargs)
         self._verts3d = xs, ys, zs
 
@@ -117,13 +156,47 @@ class Line3D(lines.Line2D):
         try:
             # If *zs* is a list or array, then this will fail and
             # just proceed to juggle_axes().
-            zs = float(zs)
-            zs = [zs for x in xs]
+            zs = np.full_like(xs, fill_value=float(zs))
         except TypeError:
             pass
         self._verts3d = juggle_axes(xs, ys, zs, zdir)
         self.stale = True
 
+    def set_data_3d(self, *args):
+        """
+        Set the x, y and z data
+
+        Parameters
+        ----------
+        x : array_like
+            The x-data to be plotted
+        y : array_like
+            The y-data to be plotted
+        z : array_like
+            The z-data to be plotted
+
+        Notes
+        -----
+        Accepts x, y, z arguments or a single array_like (x, y, z)
+        """
+        if len(args) == 1:
+            self._verts3d = args[0]
+        else:
+            self._verts3d = args
+        self.stale = True
+
+    def get_data_3d(self):
+        """
+        Get the current data
+
+        Returns
+        -------
+        verts3d : length-3 tuple or array_likes
+            The current data as a tuple or array_likes
+        """
+        return self._verts3d
+
+    @artist.allow_rasterization
     def draw(self, renderer):
         xs3d, ys3d, zs3d = self._verts3d
         xs, ys, zs = proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
@@ -133,108 +206,121 @@ class Line3D(lines.Line2D):
 
 
 def line_2d_to_3d(line, zs=0, zdir='z'):
-    '''
-    Convert a 2D line to 3D.
-    '''
+    """Convert a 2D line to 3D."""
+
     line.__class__ = Line3D
     line.set_3d_properties(zs, zdir)
 
-def path_to_3d_segment(path, zs=0, zdir='z'):
-    '''Convert a path to a 3D segment.'''
 
-    if not iterable(zs):
-        zs = np.ones(len(path)) * zs
+def _path_to_3d_segment(path, zs=0, zdir='z'):
+    """Convert a path to a 3D segment."""
 
-    seg = []
+    zs = np.broadcast_to(zs, len(path))
     pathsegs = path.iter_segments(simplify=False, curves=False)
-    for (((x, y), code), z) in zip(pathsegs, zs):
-        seg.append((x, y, z))
+    seg = [(x, y, z) for (((x, y), code), z) in zip(pathsegs, zs)]
     seg3d = [juggle_axes(x, y, z, zdir) for (x, y, z) in seg]
     return seg3d
 
+
+@cbook.deprecated("3.1")
+def path_to_3d_segment(path, zs=0, zdir='z'):
+    """Convert a path to a 3D segment."""
+    return _path_to_3d_segment(path, zs=zs, zdir=zdir)
+
+
+def _paths_to_3d_segments(paths, zs=0, zdir='z'):
+    """Convert paths from a collection object to 3D segments."""
+
+    zs = np.broadcast_to(zs, len(paths))
+    segs = [_path_to_3d_segment(path, pathz, zdir)
+            for path, pathz in zip(paths, zs)]
+    return segs
+
+
+@cbook.deprecated("3.1")
 def paths_to_3d_segments(paths, zs=0, zdir='z'):
-    '''
-    Convert paths from a collection object to 3D segments.
-    '''
+    """Convert paths from a collection object to 3D segments."""
+    return _paths_to_3d_segments(paths, zs=zs, zdir=zdir)
 
-    if not iterable(zs):
-        zs = np.ones(len(paths)) * zs
 
-    segments = []
-    for path, pathz in zip(paths, zs):
-        segments.append(path_to_3d_segment(path, pathz, zdir))
-    return segments
+def _path_to_3d_segment_with_codes(path, zs=0, zdir='z'):
+    """Convert a path to a 3D segment with path codes."""
 
-def path_to_3d_segment_with_codes(path, zs=0, zdir='z'):
-    '''Convert a path to a 3D segment with path codes.'''
-
-    if not iterable(zs):
-        zs = np.ones(len(path)) * zs
-
-    seg = []
-    codes = []
+    zs = np.broadcast_to(zs, len(path))
     pathsegs = path.iter_segments(simplify=False, curves=False)
-    for (((x, y), code), z) in zip(pathsegs, zs):
-        seg.append((x, y, z))
-        codes.append(code)
-    seg3d = [juggle_axes(x, y, z, zdir) for (x, y, z) in seg]
-    return seg3d, codes
+    seg_codes = [((x, y, z), code) for ((x, y), code), z in zip(pathsegs, zs)]
+    if seg_codes:
+        seg, codes = zip(*seg_codes)
+        seg3d = [juggle_axes(x, y, z, zdir) for (x, y, z) in seg]
+    else:
+        seg3d = []
+        codes = []
+    return seg3d, list(codes)
 
-def paths_to_3d_segments_with_codes(paths, zs=0, zdir='z'):
-    '''
+
+@cbook.deprecated("3.1")
+def path_to_3d_segment_with_codes(path, zs=0, zdir='z'):
+    """Convert a path to a 3D segment with path codes."""
+    return _path_to_3d_segment_with_codes(path, zs=zs, zdir=zdir)
+
+
+def _paths_to_3d_segments_with_codes(paths, zs=0, zdir='z'):
+    """
     Convert paths from a collection object to 3D segments with path codes.
-    '''
+    """
 
-    if not iterable(zs):
-        zs = np.ones(len(paths)) * zs
+    zs = np.broadcast_to(zs, len(paths))
+    segments_codes = [_path_to_3d_segment_with_codes(path, pathz, zdir)
+                      for path, pathz in zip(paths, zs)]
+    if segments_codes:
+        segments, codes = zip(*segments_codes)
+    else:
+        segments, codes = [], []
+    return list(segments), list(codes)
+
+
+@cbook.deprecated("3.1")
+def paths_to_3d_segments_with_codes(paths, zs=0, zdir='z'):
+    """
+    Convert paths from a collection object to 3D segments with path codes.
+    """
+    return _paths_to_3d_segments_with_codes(paths, zs=zs, zdir=zdir)
 
-    segments = []
-    codes_list = []
-    for path, pathz in zip(paths, zs):
-        segs, codes = path_to_3d_segment_with_codes(path, pathz, zdir)
-        segments.append(segs)
-        codes_list.append(codes)
-    return segments, codes_list
 
 class Line3DCollection(LineCollection):
-    '''
+    """
     A collection of 3D lines.
-    '''
-
-    def __init__(self, segments, *args, **kwargs):
-        '''
-        Keyword arguments are passed onto :func:`~matplotlib.collections.LineCollection`.
-        '''
-        LineCollection.__init__(self, segments, *args, **kwargs)
+    """
 
     def set_sort_zpos(self, val):
-        '''Set the position to use for z-sorting.'''
+        """Set the position to use for z-sorting."""
         self._sort_zpos = val
         self.stale = True
 
     def set_segments(self, segments):
-        '''
-        Set 3D segments
-        '''
+        """
+        Set 3D segments.
+        """
         self._segments3d = np.asanyarray(segments)
         LineCollection.set_segments(self, [])
 
     def do_3d_projection(self, renderer):
-        '''
+        """
         Project the points according to renderer matrix.
-        '''
+        """
         xyslist = [
             proj3d.proj_trans_points(points, renderer.M) for points in
             self._segments3d]
-        segments_2d = [list(zip(xs, ys)) for (xs, ys, zs) in xyslist]
+        segments_2d = [np.column_stack([xs, ys]) for xs, ys, zs in xyslist]
         LineCollection.set_segments(self, segments_2d)
 
         # FIXME
         minz = 1e9
-        for (xs, ys, zs) in xyslist:
+        for xs, ys, zs in xyslist:
             minz = min(minz, min(zs))
         return minz
 
+    @artist.allow_rasterization
     def draw(self, renderer, project=False):
         if project:
             self.do_3d_projection(renderer)
@@ -243,28 +329,24 @@ class Line3DCollection(LineCollection):
 
 def line_collection_2d_to_3d(col, zs=0, zdir='z'):
     """Convert a LineCollection to a Line3DCollection object."""
-    segments3d = paths_to_3d_segments(col.get_paths(), zs, zdir)
+    segments3d = _paths_to_3d_segments(col.get_paths(), zs, zdir)
     col.__class__ = Line3DCollection
     col.set_segments(segments3d)
 
 
 class Patch3D(Patch):
-    '''
+    """
     3D patch object.
-    '''
+    """
 
-    def __init__(self, *args, **kwargs):
-        zs = kwargs.pop('zs', [])
-        zdir = kwargs.pop('zdir', 'z')
+    def __init__(self, *args, zs=(), zdir='z', **kwargs):
         Patch.__init__(self, *args, **kwargs)
         self.set_3d_properties(zs, zdir)
 
     def set_3d_properties(self, verts, zs=0, zdir='z'):
-        if not iterable(zs):
-            zs = np.ones(len(verts)) * zs
-
-        self._segment3d = [juggle_axes(x, y, z, zdir) \
-                for ((x, y), z) in zip(verts, zs)]
+        zs = np.broadcast_to(zs, len(verts))
+        self._segment3d = [juggle_axes(x, y, z, zdir)
+                           for ((x, y), z) in zip(verts, zs)]
         self._facecolor3d = Patch.get_facecolor(self)
 
     def get_path(self):
@@ -275,25 +357,20 @@ class Patch3D(Patch):
 
     def do_3d_projection(self, renderer):
         s = self._segment3d
-        xs, ys, zs = list(zip(*s))
-        vxs, vys,vzs, vis = proj3d.proj_transform_clip(xs, ys, zs, renderer.M)
-        self._path2d = mpath.Path(list(zip(vxs, vys)))
+        xs, ys, zs = zip(*s)
+        vxs, vys, vzs, vis = proj3d.proj_transform_clip(xs, ys, zs, renderer.M)
+        self._path2d = mpath.Path(np.column_stack([vxs, vys]))
         # FIXME: coloring
         self._facecolor2d = self._facecolor3d
         return min(vzs)
 
-    def draw(self, renderer):
-        Patch.draw(self, renderer)
-
 
 class PathPatch3D(Patch3D):
-    '''
+    """
     3D PathPatch object.
-    '''
+    """
 
-    def __init__(self, path, **kwargs):
-        zs = kwargs.pop('zs', [])
-        zdir = kwargs.pop('zdir', 'z')
+    def __init__(self, path, *, zs=(), zdir='z', **kwargs):
         Patch.__init__(self, **kwargs)
         self.set_3d_properties(path, zs, zdir)
 
@@ -303,29 +380,38 @@ class PathPatch3D(Patch3D):
 
     def do_3d_projection(self, renderer):
         s = self._segment3d
-        xs, ys, zs = list(zip(*s))
-        vxs, vys,vzs, vis = proj3d.proj_transform_clip(xs, ys, zs, renderer.M)
-        self._path2d = mpath.Path(list(zip(vxs, vys)), self._code3d)
+        xs, ys, zs = zip(*s)
+        vxs, vys, vzs, vis = proj3d.proj_transform_clip(xs, ys, zs, renderer.M)
+        self._path2d = mpath.Path(np.column_stack([vxs, vys]), self._code3d)
         # FIXME: coloring
         self._facecolor2d = self._facecolor3d
         return min(vzs)
 
-def get_patch_verts(patch):
+
+def _get_patch_verts(patch):
     """Return a list of vertices for the path of a patch."""
     trans = patch.get_patch_transform()
-    path =  patch.get_path()
+    path = patch.get_path()
     polygons = path.to_polygons(trans)
     if len(polygons):
         return polygons[0]
     else:
         return []
 
+
+@cbook.deprecated("3.1")
+def get_patch_verts(patch):
+    """Return a list of vertices for the path of a patch."""
+    return _get_patch_verts(patch)
+
+
 def patch_2d_to_3d(patch, z=0, zdir='z'):
     """Convert a Patch to a Patch3D object."""
-    verts = get_patch_verts(patch)
+    verts = _get_patch_verts(patch)
     patch.__class__ = Patch3D
     patch.set_3d_properties(verts, z, zdir)
 
+
 def pathpatch_2d_to_3d(pathpatch, z=0, zdir='z'):
     """Convert a PathPatch to a PathPatch3D object."""
     path = pathpatch.get_path()
@@ -335,12 +421,13 @@ def pathpatch_2d_to_3d(pathpatch, z=0, zdir='z'):
     pathpatch.__class__ = PathPatch3D
     pathpatch.set_3d_properties(mpath, z, zdir)
 
-class Patch3DCollection(PatchCollection):
-    '''
-    A collection of 3D patches.
-    '''
 
-    def __init__(self, *args, **kwargs):
+class Patch3DCollection(PatchCollection):
+    """
+    A collection of 3D patches.
+    """
+
+    def __init__(self, *args, zs=0, zdir='z', depthshade=True, **kwargs):
         """
         Create a collection of flat 3D patches with its normal vector
         pointed in *zdir* direction, and located at *zs* on the *zdir*
@@ -356,14 +443,12 @@ class Patch3DCollection(PatchCollection):
         give the appearance of depth (default is *True*).
         This is typically desired in scatter plots.
         """
-        zs = kwargs.pop('zs', 0)
-        zdir = kwargs.pop('zdir', 'z')
-        self._depthshade = kwargs.pop('depthshade', True)
-        PatchCollection.__init__(self, *args, **kwargs)
+        self._depthshade = depthshade
+        super().__init__(*args, **kwargs)
         self.set_3d_properties(zs, zdir)
 
     def set_sort_zpos(self, val):
-        '''Set the position to use for z-sorting.'''
+        """Set the position to use for z-sorting."""
         self._sort_zpos = val
         self.stale = True
 
@@ -373,7 +458,7 @@ class Patch3DCollection(PatchCollection):
         self.update_scalarmappable()
         offsets = self.get_offsets()
         if len(offsets) > 0:
-            xs, ys = list(zip(*offsets))
+            xs, ys = offsets.T
         else:
             xs = []
             ys = []
@@ -386,16 +471,16 @@ class Patch3DCollection(PatchCollection):
         xs, ys, zs = self._offsets3d
         vxs, vys, vzs, vis = proj3d.proj_transform_clip(xs, ys, zs, renderer.M)
 
-        fcs = (zalpha(self._facecolor3d, vzs) if self._depthshade else
+        fcs = (_zalpha(self._facecolor3d, vzs) if self._depthshade else
                self._facecolor3d)
         fcs = mcolors.to_rgba_array(fcs, self._alpha)
         self.set_facecolors(fcs)
 
-        ecs = (zalpha(self._edgecolor3d, vzs) if self._depthshade else
+        ecs = (_zalpha(self._edgecolor3d, vzs) if self._depthshade else
                self._edgecolor3d)
         ecs = mcolors.to_rgba_array(ecs, self._alpha)
         self.set_edgecolors(ecs)
-        PatchCollection.set_offsets(self, list(zip(vxs, vys)))
+        PatchCollection.set_offsets(self, np.column_stack([vxs, vys]))
 
         if vzs.size > 0:
             return min(vzs)
@@ -404,11 +489,11 @@ class Patch3DCollection(PatchCollection):
 
 
 class Path3DCollection(PathCollection):
-    '''
+    """
     A collection of 3D paths.
-    '''
+    """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, zs=0, zdir='z', depthshade=True, **kwargs):
         """
         Create a collection of flat 3D paths with its normal vector
         pointed in *zdir* direction, and located at *zs* on the *zdir*
@@ -424,14 +509,12 @@ class Path3DCollection(PathCollection):
         give the appearance of depth (default is *True*).
         This is typically desired in scatter plots.
         """
-        zs = kwargs.pop('zs', 0)
-        zdir = kwargs.pop('zdir', 'z')
-        self._depthshade = kwargs.pop('depthshade', True)
-        PathCollection.__init__(self, *args, **kwargs)
+        self._depthshade = depthshade
+        super().__init__(*args, **kwargs)
         self.set_3d_properties(zs, zdir)
 
     def set_sort_zpos(self, val):
-        '''Set the position to use for z-sorting.'''
+        """Set the position to use for z-sorting."""
         self._sort_zpos = val
         self.stale = True
 
@@ -441,7 +524,7 @@ class Path3DCollection(PathCollection):
         self.update_scalarmappable()
         offsets = self.get_offsets()
         if len(offsets) > 0:
-            xs, ys = list(zip(*offsets))
+            xs, ys = offsets.T
         else:
             xs = []
             ys = []
@@ -454,21 +537,18 @@ class Path3DCollection(PathCollection):
         xs, ys, zs = self._offsets3d
         vxs, vys, vzs, vis = proj3d.proj_transform_clip(xs, ys, zs, renderer.M)
 
-        fcs = (zalpha(self._facecolor3d, vzs) if self._depthshade else
+        fcs = (_zalpha(self._facecolor3d, vzs) if self._depthshade else
                self._facecolor3d)
         fcs = mcolors.to_rgba_array(fcs, self._alpha)
         self.set_facecolors(fcs)
 
-        ecs = (zalpha(self._edgecolor3d, vzs) if self._depthshade else
+        ecs = (_zalpha(self._edgecolor3d, vzs) if self._depthshade else
                self._edgecolor3d)
         ecs = mcolors.to_rgba_array(ecs, self._alpha)
         self.set_edgecolors(ecs)
-        PathCollection.set_offsets(self, list(zip(vxs, vys)))
+        PathCollection.set_offsets(self, np.column_stack([vxs, vys]))
 
-        if vzs.size > 0 :
-            return min(vzs)
-        else :
-            return np.nan
+        return np.min(vzs) if vzs.size else np.nan
 
 
 def patch_collection_2d_to_3d(col, zs=0, zdir='z', depthshade=True):
@@ -478,15 +558,15 @@ def patch_collection_2d_to_3d(col, zs=0, zdir='z', depthshade=True):
     (or a :class:`~matplotlib.collections.PathCollection` into a
     :class:`Path3DCollection` object).
 
-    Keywords:
-
-    *za*            The location or locations to place the patches in the
-                    collection along the *zdir* axis. Defaults to 0.
-
-    *zdir*          The axis in which to place the patches. Default is "z".
-
-    *depthshade*    Whether to shade the patches to give a sense of depth.
-                    Defaults to *True*.
+    Parameters
+    ----------
+    za
+        The location or locations to place the patches in the collection along
+        the *zdir* axis. Default: 0.
+    zdir
+        The axis in which to place the patches. Default: "z".
+    depthshade
+        Whether to shade the patches to give a sense of depth. Default: *True*.
 
     """
     if isinstance(col, PathCollection):
@@ -498,12 +578,12 @@ def patch_collection_2d_to_3d(col, zs=0, zdir='z', depthshade=True):
 
 
 class Poly3DCollection(PolyCollection):
-    '''
+    """
     A collection of 3D polygons.
-    '''
+    """
 
-    def __init__(self, verts, *args, **kwargs):
-        '''
+    def __init__(self, verts, *args, zsort='average', **kwargs):
+        """
         Create a Poly3DCollection.
 
         *verts* should contain 3D coordinates.
@@ -513,9 +593,8 @@ class Poly3DCollection(PolyCollection):
 
         Note that this class does a bit of magic with the _facecolors
         and _edgecolors properties.
-        '''
-        zsort = kwargs.pop('zsort', True)
-        PolyCollection.__init__(self, verts, *args, **kwargs)
+        """
+        super().__init__(verts, *args, **kwargs)
         self.set_zsort(zsort)
         self._codes3d = None
 
@@ -526,43 +605,41 @@ class Poly3DCollection(PolyCollection):
     }
 
     def set_zsort(self, zsort):
-        '''
-        Set z-sorting behaviour:
-            boolean: if True use default 'average'
-            string: 'average', 'min' or 'max'
-        '''
+        """
+        Sets the calculation method for the z-order.
 
+        Parameters
+        ----------
+        zsort : {'average', 'min', 'max'}
+            The function applied on the z-coordinates of the vertices in the
+            viewer's coordinate system, to determine the z-order.  *True* is
+            deprecated and equivalent to 'average'.
+        """
         if zsort is True:
+            cbook.warn_deprecated(
+                "3.1", message="Passing True to mean 'average' for set_zsort "
+                "is deprecated and support will be removed in Matplotlib 3.3; "
+                "pass 'average' instead.")
             zsort = 'average'
-
-        if zsort is not False:
-            if zsort in self._zsort_functions:
-                zsortfunc = self._zsort_functions[zsort]
-            else:
-                return False
-        else:
-            zsortfunc = None
-
-        self._zsort = zsort
+        self._zsortfunc = self._zsort_functions[zsort]
         self._sort_zpos = None
-        self._zsortfunc = zsortfunc
         self.stale = True
 
     def get_vector(self, segments3d):
-        """Optimize points for projection"""
+        """Optimize points for projection."""
         si = 0
         ei = 0
         segis = []
         points = []
         for p in segments3d:
             points.extend(p)
-            ei = si+len(p)
+            ei = si + len(p)
             segis.append((si, ei))
             si = ei
 
-        if len(segments3d) > 0 :
-            xs, ys, zs = list(zip(*points))
-        else :
+        if len(segments3d):
+            xs, ys, zs = zip(*points)
+        else:
             # We need this so that we can skip the bad unpacking from zip()
             xs, ys, zs = [], [], []
 
@@ -571,13 +648,14 @@ class Poly3DCollection(PolyCollection):
         self._segis = segis
 
     def set_verts(self, verts, closed=True):
-        '''Set 3D vertices.'''
+        """Set 3D vertices."""
         self.get_vector(verts)
         # 2D verts will be updated at draw time
-        PolyCollection.set_verts(self, [], closed)
+        PolyCollection.set_verts(self, [], False)
+        self._closed = closed
 
     def set_verts_and_codes(self, verts, codes):
-        '''Sets 3D vertices with path codes'''
+        """Sets 3D vertices with path codes."""
         # set vertices with closed=False to prevent PolyCollection from
         # setting path codes
         self.set_verts(verts, closed=False)
@@ -589,27 +667,27 @@ class Poly3DCollection(PolyCollection):
         # just in case it is a scalarmappable with a colormap.
         self.update_scalarmappable()
         self._sort_zpos = None
-        self.set_zsort(True)
-        self._facecolors3d = PolyCollection.get_facecolors(self)
-        self._edgecolors3d = PolyCollection.get_edgecolors(self)
+        self.set_zsort('average')
+        self._facecolors3d = PolyCollection.get_facecolor(self)
+        self._edgecolors3d = PolyCollection.get_edgecolor(self)
         self._alpha3d = PolyCollection.get_alpha(self)
         self.stale = True
 
-    def set_sort_zpos(self,val):
-        '''Set the position to use for z-sorting.'''
+    def set_sort_zpos(self, val):
+        """Set the position to use for z-sorting."""
         self._sort_zpos = val
         self.stale = True
 
     def do_3d_projection(self, renderer):
-        '''
+        """
         Perform the 3D projection for this object.
-        '''
+        """
         # FIXME: This may no longer be needed?
         if self._A is not None:
             self.update_scalarmappable()
             self._facecolors3d = self._facecolors
 
-        txs, tys, tzs = proj3d.proj_transform_vec(self._vec, renderer.M)
+        txs, tys, tzs = proj3d._proj_transform_vec(self._vec, renderer.M)
         xyzlist = [(txs[si:ei], tys[si:ei], tzs[si:ei])
                    for si, ei in self._segis]
 
@@ -624,22 +702,19 @@ class Poly3DCollection(PolyCollection):
             else:
                 cedge = cedge.repeat(len(xyzlist), axis=0)
 
-        # if required sort by depth (furthest drawn first)
-        if self._zsort:
-            indices = range(len(xyzlist))
-            z_segments_2d = [(self._zsortfunc(zs), list(zip(xs, ys)), fc, ec,
-                              idx) for (xs, ys, zs), fc, ec, idx in
-                             zip(xyzlist, cface, cedge, indices)]
-            z_segments_2d.sort(key=lambda x: x[0], reverse=True)
-        else:
-            raise ValueError("whoops")
+        # sort by depth (furthest drawn first)
+        z_segments_2d = sorted(
+            ((self._zsortfunc(zs), np.column_stack([xs, ys]), fc, ec, idx)
+             for idx, ((xs, ys, zs), fc, ec)
+             in enumerate(zip(xyzlist, cface, cedge))),
+            key=lambda x: x[0], reverse=True)
 
         segments_2d = [s for z, s, fc, ec, idx in z_segments_2d]
         if self._codes3d is not None:
             codes = [self._codes3d[idx] for z, s, fc, ec, idx in z_segments_2d]
             PolyCollection.set_verts_and_codes(self, segments_2d, codes)
         else:
-            PolyCollection.set_verts(self, segments_2d)
+            PolyCollection.set_verts(self, segments_2d, self._closed)
 
         self._facecolors2d = [fc for z, s, fc, ec, idx in z_segments_2d]
         if len(self._edgecolors3d) == len(cface):
@@ -650,32 +725,31 @@ class Poly3DCollection(PolyCollection):
         # Return zorder value
         if self._sort_zpos is not None:
             zvec = np.array([[0], [0], [self._sort_zpos], [1]])
-            ztrans = proj3d.proj_transform_vec(zvec, renderer.M)
+            ztrans = proj3d._proj_transform_vec(zvec, renderer.M)
             return ztrans[2][0]
-        elif tzs.size > 0 :
+        elif tzs.size > 0:
             # FIXME: Some results still don't look quite right.
             #        In particular, examine contourf3d_demo2.py
             #        with az = -54 and elev = -45.
             return np.min(tzs)
-        else :
+        else:
             return np.nan
 
     def set_facecolor(self, colors):
         PolyCollection.set_facecolor(self, colors)
         self._facecolors3d = PolyCollection.get_facecolor(self)
-    set_facecolors = set_facecolor
 
     def set_edgecolor(self, colors):
         PolyCollection.set_edgecolor(self, colors)
         self._edgecolors3d = PolyCollection.get_edgecolor(self)
-    set_edgecolors = set_edgecolor
 
     def set_alpha(self, alpha):
         """
-        Set the alpha tranparencies of the collection.  *alpha* must be
-        a float or *None*.
+        Set the alpha transparencies of the collection.
 
-        ACCEPTS: float or None
+        Parameters
+        ----------
+        alpha : float or None
         """
         if alpha is not None:
             try:
@@ -684,7 +758,7 @@ class Poly3DCollection(PolyCollection):
                 raise TypeError('alpha must be a float or None')
         artist.Artist.set_alpha(self, alpha)
         try:
-            self._facecolors = mcolors.to_rgba_array(
+            self._facecolors3d = mcolors.to_rgba_array(
                 self._facecolors3d, self._alpha)
         except (AttributeError, TypeError, IndexError):
             pass
@@ -695,26 +769,22 @@ class Poly3DCollection(PolyCollection):
             pass
         self.stale = True
 
-    def get_facecolors(self):
+    def get_facecolor(self):
         return self._facecolors2d
-    get_facecolor = get_facecolors
 
-    def get_edgecolors(self):
+    def get_edgecolor(self):
         return self._edgecolors2d
-    get_edgecolor = get_edgecolors
-
-    def draw(self, renderer):
-        return Collection.draw(self, renderer)
 
 
 def poly_collection_2d_to_3d(col, zs=0, zdir='z'):
     """Convert a PolyCollection to a Poly3DCollection object."""
-    segments_3d, codes = paths_to_3d_segments_with_codes(col.get_paths(),
-                                                         zs, zdir)
+    segments_3d, codes = _paths_to_3d_segments_with_codes(
+            col.get_paths(), zs, zdir)
     col.__class__ = Poly3DCollection
     col.set_verts_and_codes(segments_3d, codes)
     col.set_3d_properties()
 
+
 def juggle_axes(xs, ys, zs, zdir):
     """
     Reorder coordinates so that 2D xs, ys can be plotted in the plane
@@ -730,6 +800,7 @@ def juggle_axes(xs, ys, zs, zdir):
     else:
         return xs, ys, zs
 
+
 def rotate_axes(xs, ys, zs, zdir):
     """
     Reorder coordinates so that the axes are rotated with zdir along
@@ -749,45 +820,35 @@ def rotate_axes(xs, ys, zs, zdir):
     else:
         return xs, ys, zs
 
-def iscolor(c):
-    try:
-        if len(c) == 4 or len(c) == 3:
-            if iterable(c[0]):
-                return False
-            if hasattr(c[0], '__float__'):
-                return True
-    except:
-        return False
-    return False
 
+def _get_colors(c, num):
+    """Stretch the color argument to provide the required number *num*."""
+    return np.broadcast_to(
+        mcolors.to_rgba_array(c) if len(c) else [0, 0, 0, 0],
+        (num, 4))
+
+
+@cbook.deprecated("3.1")
 def get_colors(c, num):
-    """Stretch the color argument to provide the required number num"""
+    """Stretch the color argument to provide the required number *num*."""
+    return _get_colors(c, num)
 
-    if type(c) == type("string"):
-        c = mcolors.to_rgba(c)
 
-    if iscolor(c):
-        return [c] * num
-    if len(c) == num:
-        return c
-    elif iscolor(c):
-        return [c] * num
-    elif len(c) == 0: #if edgecolor or facecolor is specified as 'none'
-        return [[0,0,0,0]] * num
-    elif iscolor(c[0]):
-        return [c[0]] * num
-    else:
-        raise ValueError('unknown color format %s' % c)
-
-def zalpha(colors, zs):
-    """Modify the alphas of the color list according to depth"""
+def _zalpha(colors, zs):
+    """Modify the alphas of the color list according to depth."""
     # FIXME: This only works well if the points for *zs* are well-spaced
     #        in all three dimensions. Otherwise, at certain orientations,
     #        the min and max zs are very close together.
     #        Should really normalize against the viewing depth.
-    colors = get_colors(colors, len(zs))
-    if zs.size > 0 :
-        norm = Normalize(min(zs), max(zs))
-        sats = 1 - norm(zs) * 0.7
-        colors = [(c[0], c[1], c[2], c[3] * s) for c, s in zip(colors, sats)]
-    return colors
+    if len(zs) == 0:
+        return np.zeros((0, 4))
+    norm = Normalize(min(zs), max(zs))
+    sats = 1 - norm(zs) * 0.7
+    rgba = np.broadcast_to(mcolors.to_rgba_array(colors), (len(zs), 4))
+    return np.column_stack([rgba[:, :3], rgba[:, 3] * sats])
+
+
+@cbook.deprecated("3.1")
+def zalpha(colors, zs):
+    """Modify the alphas of the color list according to depth."""
+    return _zalpha(colors, zs)
